library("RMVL")
#
# In this example we use an input file "points_of_interest.csv" which contains coordinates for several points of interest (f0, ra, dec)
# We use previously constructed spatial index to find points in the O3a atlas that are closest to points of interest
# Results are printed in the end of the file
#

# O3a_atlas.mvl contains results of O3a analysis
Matlas<-mvl_open("O3a_2_atlas.mvl")

# O3a_spatial_index.mvl is created by script create_spatial_index.R from O3a_atlas.mvl
Mspatial<-mvl_open("O3a_2_spatial_index.mvl")

skymaps_band_start<-Mspatial$skymaps_band_start
x0<-Mspatial$x0
x1<-Mspatial$x1
x2<-Mspatial$x2

# User provided files. Example included.
# For very large files, it might be better to store data in MVL format
points_of_interest<-read.table("points_of_interest.csv", header=TRUE)

f0poi<-points_of_interest[,"frequency"]
x0poi<-cos(points_of_interest[,"ra"])*cos(points_of_interest[,"dec"])*f0poi/7.5
x1poi<-sin(points_of_interest[,"ra"])*cos(points_of_interest[,"dec"])*f0poi/7.5
x2poi<-sin(points_of_interest[,"dec"])*f0poi/7.5



f_closest<-function(i, idx) {
	f0<-f0poi[i]
	# Find frequency bands that cover our point of interest
	fstart<-skymaps_band_start[idx]
	Ffreq<-fstart<=f0 & (fstart+0.045)>=f0
	idx0<-idx[Ffreq]
	if(length(idx0)<1)return(NA)
	
	# Find closest (on the sky) point to our point of interest
	d<-(x0[idx0]-x0poi[i])^2+(x1[idx0]-x1poi[i])^2+(x2[idx0]-x2poi[i])^2
	return(idx0[which.min(d)])
	}

closest_indices<-unlist(mvl_index_lapply(Mspatial$spatial_index, list(f0poi-0.045*0.5, x0poi, x1poi, x2poi), f_closest))

Ffound<-!is.na(closest_indices)
i0<-1:length(f0poi)
ipoi<-i0[Ffound]
iatlas<-closest_indices[Ffound]

df<-data.frame(points_of_interest[ipoi,,drop=FALSE], Matlas$skymaps[iatlas, c("ul", "ul_circ", "snr", "snr_frequency", "ra", "dec", "idx"),recurse=TRUE], band_start=skymaps_band_start[iatlas])
names(df)<-gsub(".1", "_falcon", names(df))

print(df)

# You can write data to a text file with the following command:
# write.table(df, "spatial_index_example1_output.csv", col.names=TRUE, row.names=FALSE, sep="\t")
#




