library("RMVL")
#
# In this example we retrieve upper limits and signal to noise ratios for a particular sky location, using previously constructed spatial index.
#

# O3a_atlas.mvl contains results of O3a analysis
Matlas<-mvl_open("O3a_2_atlas.mvl")

# O3a_spatial_index.mvl is created by the script create_spatial_index.R from O3a_atlas.mvl
Mspatial<-mvl_open("O3a_2_spatial_index.mvl")

skymaps_band_start<-Mspatial$skymaps_band_start
x0<-Mspatial$x0
x1<-Mspatial$x1
x2<-Mspatial$x2

#
# Coordinates in radians, J2000
#
RA<-0.0
DEC<-0.0

# Search all frequencies
f0poi<-sort(unique(Matlas$parameters[,"band_start"][]))

# Rescale for spatial index
x0poi<-cos(RA)*cos(DEC)*f0poi/7.5
x1poi<-sin(RA)*cos(DEC)*f0poi/7.5
x2poi<-sin(DEC)*f0poi/7.5



i0<-1:length(f0poi)

f_closest<-function(i, idx) {
	f0<-f0poi[i]
	# Find frequency bands that cover our point of interest
	fstart<-skymaps_band_start[idx]
	Ffreq<-(fstart-0.5*0.045)<=f0 & (fstart+0.5*0.045)>=f0
	idx0<-idx[Ffreq]
	if(length(idx0)<1)return(NA)
	
	# Find closest (on the sky) point to our point of interest
	d<-(x0[idx0]-x0poi[i])^2+(x1[idx0]-x1poi[i])^2+(x2[idx0]-x2poi[i])^2
	return(idx0[which.min(d)])
	}


closest_indices<-unlist(mvl_index_lapply(Mspatial$spatial_index, list(f0poi, x0poi, x1poi, x2poi), f_closest))

Ffound<-!is.na(closest_indices)
ipoi<-i0[Ffound]
iatlas<-closest_indices[Ffound]

df<-data.frame(Matlas$skymaps[iatlas, c("ul", "ul_circ", "snr", "snr_frequency", "ra", "dec", "idx"),recurse=TRUE], band_start=skymaps_band_start[iatlas])
names(df)<-gsub(".1", "_falcon", names(df))

print(df[1:10,])

## Uncomment the line below to write out results
#write.table(df, "spatial_index_example2_output.csv", col.names=TRUE, row.names=FALSE, sep="\t")





